#=============================================================================
# Doloforge::Service::Cron : prepare cron/(daily,weekly,...) directories
#
# Create the directories in which developers will create cron scripts.
#
# Full help :
#   perldoc Doloforge::Service::Cron
#=============================================================================


package Doloforge::Service::Cron;

use strict;

# This base class implements the constructor and help methods in a generic way.
use base qw(Doloforge::Service);
use File::Path;


#=============================================================================
=head1 NAME

Doloforge::Service::Cron : prepare cron/(daily,weekly,...) directories

=head1 SYNOPSIS

    # Update the full configuration, running each client/project/service update
    doloforge config --update

=head1 DESCRIPTION

Todo

=cut


#=============================================================================
# Method update
#
# Update the service configuration on the host, depending on
# client/project/service config files...
# This method receives :
#   option: context options passesd on the command-line (quite useless there)
#   client_conf: client configuration file contents (client.$name.yaml)
#   project_conf: project configuration file contents (project.$name.client.$name.yaml)

sub update {
    my $self = shift;

    my ($option, $client_conf, $project_conf) = @_;

    # Copy the service skeleton directory into the project local and shared directory.
    foreach my $dirid qw(shared local) {
        $self->skel_copy(
            "$self->{global}->{dir}->{skel}/service/$dirid/$self->{name}",                  # Skeleton directory to copy (skel/service/[local|shared]/[service]/)
            "$self->{global}->{dir}->{$dirid}/$client_conf->{name}/$project_conf->{name}",  # Where to copy it
            $self->{config}->{developerdirs},                                               # reference to the developers directories, that will have to be chown'ed to the following ownership
            $client_conf->{name}, $client_conf->{name},                                     # user/group id of the client who own this project. Developers of the same group will have write-access to files there.
        ) if ((($dirid eq 'shared') and $self->has_role('nfs_role'))                        # only create shared storage on the NFS server
           or (($dirid eq 'local') and $self->has_role('appserv_role')));                   # only create local storage on application servers
    }
}


#=============================================================================
# Method finish
#
# Called when the whole processing is finished, when it's time to do one-time things,
# like creating the lock directory...

sub finish {
    my $self = shift;

    # Create the lock directory on the NFS server.
    # This lock dir should belong to user "nobody", which is the ID the client user "root" have when it connects to a root_squash NFS share
    if ($self->has_role('nfs_role')) {
        my $lockdir = "$self->{global}->{dir}->{shared}/.doloforge/$self->{name}/lock";
        unless (-d $lockdir) {
            $self->info("Creating $lockdir lock directory, with owner 'nobody'");
            mkpath($lockdir);
            # Change owner to numeric ID of 'nobody' (which must exist on every system), without changing group
            chown(scalar(getpwnam('nobody')), -1, $lockdir);
        }
    }
}


#=============================================================================
# Method cron_run
#
# Run the regular cron tasks on a per-project basis, on the application servers,
# which is mainly DB backup for every project that uses this RDBMS.

sub cron_run {
    my $self = shift;

    my ($client, $project, $date, $interval) = @_;

    # Run all the jobs in a controled environment...
    $self->run_job($client, $project, "cron/$interval/*", [$date, $interval], undef);
}



1;
__END__

=head1 AUTHOR

Nicolas Huillard <F<nicolas@huillard.net>>, Dolomède

=head1 SEE ALSO

doloforge(1)

=cut
